package org.warp.jcwdb;

import it.unimi.dsi.fastutil.ints.Int2ObjectMap;
import it.unimi.dsi.fastutil.ints.Int2ObjectOpenHashMap;
import it.unimi.dsi.fastutil.objects.Object2ObjectMap;
import it.unimi.dsi.fastutil.objects.Object2ObjectOpenHashMap;

public class TypesManager {
	private final Int2ObjectMap<DBTypeParser<?>> types;
	private final Object2ObjectMap<Class<?>, DBTypeParser<?>> typesByClass;
	private DBTypedObjectParser<?> fallbackParser;

	public TypesManager(JCWDatabase db) {
		types = new Int2ObjectOpenHashMap<>();
		typesByClass = new Object2ObjectOpenHashMap<>();
		DBStandardTypes.registerStandardTypes(db, this);
	}

	public <T> void registerType(Class<T> clazz, int type, DBTypeParser<T> parser) {
		this.types.put(type, parser);
		this.typesByClass.put(clazz, parser);
	}

	/**
	 * Use this method with the most used classes to save disk space.
	 * @param clazz
	 * @param id
	 * @param <T>
	 */
	public <T> void registerGenericClass(Class<T> clazz, int id) {
		this.fallbackParser.registerClass(clazz, id);
	}

	public void registerTypeFallback(DBTypedObjectParser<?> parser) {
		this.fallbackParser = parser;
	}

	public <T> DBTypeParser<T> get(int type) {
		if (types.containsKey(type) == false) {
			if (fallbackParser == null) {
				throw new NoParserFoundException("The type " + type + " can't be parsed.");
			} else {
				return fallbackParser.cast();
			}
		}
		return types.get(type).cast();
	}

	public <T> DBTypeParser<T> get(Class<T> type) {
		if (typesByClass.containsKey(type) == false) {
			if (fallbackParser == null) {
				throw new NoParserFoundException("The class " + type.getSimpleName() + " can't be parsed.");
			} else {
				return fallbackParser.cast();
			}
		}
		return typesByClass.get(type).cast();
	}
}
